#! /usr/bin/env python3
# -*- coding: utf-8 -*-

# =====================================================================
# Copyright (C) 2018-2019 by Owl Data
# author: Danny, Destiny

# =====================================================================

# 各種欄位表
colist_dict = {
    'ssp':['日期', '股票名稱', '開盤價', '最高價', 
           '最低價', '收盤價', '漲跌', '漲幅(%)', 
           '成交量', '成交筆數', '成交金額(千)', '均張', 
           '均價', '成交量(股)'],
    
    'msp':['股票代號', '股票名稱', '日期', '開盤價',
            '最高價', '最低價', '收盤價', '漲跌', '漲幅(%)', 
            '成交量', '成交筆數', '成交金額(千)', '均張', 
            '均價', '成交量(股)'],
    
    'fis':{
           'y':['年度', '流動資產', '非流動資產', '資產總計', '流動負債', 
                '非流動負債', '負債總計', '權益總計', '公告每股淨值', 
                '營業收入(千)', '營業成本(千)', '營業毛利(千)', 
                '營業費用(千)', '營業利益(千)', '營業外收入及支出(千)',
                '稅前純益(千)', '所得稅(千)', '稅後純益歸屬(千)', 
                '每股盈餘(元)', '營業活動現金流量(千)', 
                '投資活動現金流量(千)', '籌資活動現金流量(千)', 
                '本期現金及約當現金增減數(千)', '期末現金及約當現金餘額(千)',
                '自由現金流量(千)'],
           
           'm':['年月', '單月合併營收(千)', '去年同期(千)', 
                '單月合併營收年成長(%)', '單月合併營收月變動(%)', 
                '累計合併營收(千)', '去年同期(千)1', '累計合併營收成長(%)'],
           
           'q':['年季', '流動資產', '非流動資產', '資產總計', '流動負債', 
                '非流動負債', '負債總計', '權益總計', '公告每股淨值', '營業收入(千)', 
                '營業成本(千)', '營業毛利(千)', '營業費用(千)', '營業利益(千)', 
                '營業外收入及支出(千)', '稅前純益(千)', '所得稅(千)', 
                '稅後純益歸屬(千)', '每股盈餘(元)', '營業活動現金流量(千)', 
                '投資活動現金流量(千)', '籌資活動現金流量(千)', 
                '本期現金及約當現金增減數(千)', '期末現金及約當現金餘額(千)', 
                '自由現金流量(千)']
           },

    'fim':{
           'y':['股票代號', '股票名稱', '年度', '流動資產', '非流動資產', 
                '資產總計', '流動負債', '非流動負債', '負債總計', '權益總計',
                 '公告每股淨值', '營業收入(千)', '營業成本(千)', 
                 '營業毛利(千)', '營業費用(千)', '營業利益(千)', 
                 '營業外收入及支出(千)', '稅前純益(千)', '所得稅(千)', 
                 '稅後純益歸屬(千)', '每股盈餘(元)', '營業活動現金流量(千)', 
                 '投資活動現金流量(千)', '籌資活動現金流量(千)', 
                 '本期現金及約當現金增減數(千)', '期末現金及約當現金餘額(千)', 
                 '自由現金流量(千)'],
           
           'm':['股票代號', '股票名稱', '年月', '單月合併營收(千)', 
                '去年同期(千)', '單月合併營收年成長(%)',
                '單月合併營收月變動(%)', '累計合併營收(千)', 
                '去年同期(千)1', '累計合併營收成長(%)'],
           
           'q':['股票代號', '股票名稱', '年季', '流動資產', '非流動資產', 
                '資產總計', '流動負債', '非流動負債', '負債總計', 
                '權益總計', '公告每股淨值', '營業收入(千)', '營業成本(千)', 
                '營業毛利(千)', '營業費用(千)', '營業利益(千)', 
                '營業外收入及支出(千)', '稅前純益(千)', '所得稅(千)',
                '稅後純益歸屬(千)', '每股盈餘(元)', '營業活動現金流量(千)',
                '投資活動現金流量(千)', '籌資活動現金流量(千)', 
                '本期現金及約當現金增減數(千)', '期末現金及約當現金餘額(千)',
                '自由現金流量(千)']
           },

    'chs':['日期', '法人買賣超', '法人買賣超金額(千)', '外資買賣超', 
           '外資買賣超金額(千)', '投信買賣超', '投信買賣超金額(千)', 
           '自營買賣超', '自營買賣超金額(千)', '融資增減', '融資餘額', 
           '融資使用率', '融券增減', '融券餘額', '融券使用率', '券資比', '當沖比率'],
    
    'chm':['股票代號', '股票名稱', '法人買賣超', '法人買賣超金額(千)', 
           '外資買賣超', '外資買賣超金額(千)', '投信買賣超', '投信買賣超金額(千)', 
           '自營買賣超', '自營買賣超金額(千)', '融資增減', '融資餘額', '融資使用率',
            '融券增減', '融券餘額', '融券使用率', '券資比', '當沖比率'],
    
    'tis':['日期', 'K(9)', 'D(9)', 'RSI(5)', 'RSI(10)', 'DIF', 
           'MACD', 'DIF-MACD', 'W%R(5)', 'W%R(10)', '+DI(14)', 
           '-DI(14)', 'ADX(14)'],
    
    'tim':['股票代號', '股票名稱', '日期', 'K(9)', 'D(9)', 'RSI(5)',
            'RSI(10)', 'DIF', 'MACD', 'DIF-MACD', 'W%R(5)',
             'W%R(10)', '+DI(14)', '-DI(14)', 'ADX(14)'],
    
    'cim':['股票代號', '股票名稱', '中文簡稱', '英文簡稱', '公司名稱', 
           '英文名稱', '地址', '電話', '傳真機號碼', '實收資本額(百萬)', 
           '成立日期', '上市日期', '上櫃日期', '興櫃日期', '公發日期', 
           '董事長', '總經理', '發言人', '上市上櫃', '產業名稱', '統一編號', 
           '交易所公告股本(千)'],
    
    'dps':['年度', '現金股利合計', '股票股利合計', '股利合計', 
           '現金股利殖利率(%)', '盈餘配息', '公積配息', '盈餘配股', '公積配股'],
    
    'dpm':['股票代號', '股票名稱', '年度', '現金股利合計', 
           '股票股利合計', '股利合計', '現金股利殖利率(%)', 
           '盈餘配息', '公積配息', '盈餘配股', '公積配股'],
    
    'edps':['年度', '股東會日期', '停止融券起始日', '融券回補日',
             '停止融券終迄日', '停止融資起始日', '停止融資終迄日', 
             '停止過戶起', '停止過戶迄', '最後過戶日'],
    
    'edpm':['股票代號', '股票名稱', '年度', '股東會日期', '停止融券起始日', 
            '融券回補日', '停止融券終迄日', '停止融資起始日', '停止融資終迄日',
             '停止過戶起', '停止過戶迄', '最後過戶日'],
    
    'tsp':['股票代號', '股票名稱', '時間', '成交價', '漲跌', '漲跌幅',
            '總量', '開盤價', '最高價', '最低價', '成交量']
}
